%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Code to simulate the true data from skewt-t Factor copula  model and then backtest Value-at-Risk forecasts
% based on correctly specified or misspecified factor copula models (Simulation Study 5.2).
%
%  Fang Duan
%
%  Last Update on December 2018
%
% This code was developed for the following paper:
%
% F. Duan, H. Manner and D. Wied, 2019, "Model and Moment Selection in Factor Copula Models".
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear
addmypath;
%% Parameter setting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
T = 1500;
N = 5; % N= 10
S = 25;
SizeMonteCarlo = 100000;
R = 100;

EstimationWindowSize = 1000;
TestWindowStart = EstimationWindowSize + 1;
TestWindow = TestWindowStart : T;
TestWindowSize = T - EstimationWindowSize;

% true DGP
simfunc_true = 'simfunc_skewtt';
theta_null = [1 0.25 -0.5];
dpd = 'Equidep_theta';

% moment function
momfunc = 'momfunc_Equidep';
mom = 'tail';

if strcmp(mom, 'tail')
    q = [0.05,0.1,0.9,0.95]; % quantiles at both tails + rho_S
elseif strcmp(mom, 'center')
    q = [0.25,0.45,0.55,0.75]; % quantiles at center area
elseif strcmp(mom, 'lower')
    q = [0.01,0.05,0.1,0.15]; % quantiles at lower tail
elseif strcmp(mom, 'left')
    q = [0.01,0.05,0.1,0.15]; % quantiles at lower tail + rho_S
else 
    q = [0.05,0.1,0.25,0.45,0.55,0.75,0.9,0.95]; % whole set of quantiles
end    

if strcmp(momfunc,'momfunc_Equidep_lower')
    W = eye(length(q));
else
    W = eye(length(q)+1);
end
phi= [0.01, 0.05, 0.05, 0.10, 0.85]; % [\phi_0,\phi_1,\omega,\alpha,\beta]

simfunc_est = 'simfunc_gaussian'; % factor copula used in SMM: 'simfunc_tt', 'simfunc_skewtt', 'simfunc_skewnn', 'simfunc_gaussian'

% estimation setting in SMM
options = optimset('MaxIter',1000);
if strcmp(simfunc_est,'simfunc_gaussian')
    thetahat = zeros(R,1);
    theta_guess = 2;
    LB = 0;
    UB = 5;
elseif strcmp(simfunc_est,'simfunc_tt')
    thetahat = zeros(R,2);
    theta_guess = [2 0.25];
    LB = [0 0.01];
    UB = [5 0.49];
elseif strcmp(simfunc_est,'simfunc_skewtt')
    thetahat = zeros(R,3);
    theta_guess = [2 0.25 0];
    LB = [0 0.01 -0.99];
    UB = [5 0.49 0.99];
elseif strcmp(simfunc_est,'simfunc_skewnn')
    thetahat = zeros(R,2);
    theta_guess = [2 0];
    LB = [0 -0.99];
    UB = [5 0.99];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% VaR forecasting
Jstat = NaN(R,1);
ht_one = NaN(TestWindowSize,N);
pVaR = [0.05 0.01];
VaR = NaN(TestWindowSize,length(pVaR),R); % VaR_0.05 and VaR_0.01 for each t in test window 

v = NaN(R,2);
coverage = NaN(R,2);
stat_kupiec = NaN(R,2);
pval_kupiec = NaN(R,2);

for index = 1:R
    disp(strcat('Round:',num2str(index)))
    seed = index;
    other = [T N seed];
    other2 = [S*EstimationWindowSize N 2*seed];
    [data0, uresid] = simulate_return(simfunc_true, theta_null, other, dpd, phi); % genenrate the true DGP with AR(1)-GARCH(1,1) margins, data0 denotes Y
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % We firstly estimate copula parameters once (based on samples in Estimation window), since we assume copula parameters are constant over time (indeed copula parameters in true DGP are constant)
    % Then in the test window, we estimate conditional variance at each time
    % point t in the test window

    % 1. Estimate copula params/Jstat in EstimationWindow
    data_est = data0(1:EstimationWindowSize,:);
    data_resid = NaN(EstimationWindowSize, N); % AR-GARCH residuals
    data_param = NaN(length(phi),N); % AR_GARCH parameters
    data_uresid = NaN(EstimationWindowSize, N); % AR-GARCH PIT residuals
    ht_est = NaN(EstimationWindowSize,N); % conditional variances in GARCH
    startval = phi;
    for dim = 1:N 
       [~,param_temp,~,~,resid_temp,ht_temp,~] = fitARMAGARCH(data_est(:,dim),[1 0 1 0 1],'NORMAL',startval);
       data_param(:,dim) = param_temp';
       data_resid(:,dim) = resid_temp;
       ht_est(:,dim) = ht_temp;
       data_uresid(:,dim) = tiedrank(resid_temp)/(EstimationWindowSize+1); % non-parametric transformation
    end
    [thetahat(index,:),~] = fminsearchbnd('objfunc_SMM',theta_guess,LB,UB,options,simfunc_est,other2,dpd,momfunc,data_uresid,W,q);
    [~, ghat] = objfunc_SMM(thetahat(index,:), simfunc_est, other2, dpd, momfunc, data_uresid, W, q);
    Jstat(index,1) = min(EstimationWindowSize, S*EstimationWindowSize) * ghat' * W * ghat;
    
    % 2. Generate uniform standardized residuals/innovations(eta) based on
    % estimated copula parameter and then obtain residual
    data_uresid_sim = feval(simfunc_est,thetahat(index,:),[SizeMonteCarlo N 2*seed],dpd);
    data_resid_sim = NaN(SizeMonteCarlo,N);
    for d = 1:N
        data_resid_sim(:,d) = norminv(data_uresid_sim(:,d),0,1);
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % 3. Estimate conditional volatilities in margins with rolling window,
    % and obtain the distribution of simulated portfolio return
    for t = TestWindow
        i = t - TestWindowStart + 1;
        disp(strcat('Test Sample:', num2str(i)))
        EstimationWindow = t - EstimationWindowSize : t - 1;
        data_est = data0(EstimationWindow,:);
        data_resid = NaN(EstimationWindowSize, N); % AR-GARCH residuals
        data_param = NaN(length(phi),N); % AR-GARCH parameters
        data_uresid = NaN(EstimationWindowSize, N); % AR-GARCH PIT residuals
        ht_est = NaN(EstimationWindowSize,N); % conditional variances in GARCH
        startval = phi;

        for dim = 1:N 
           [~,param_temp,~,~,resid_temp,ht_temp,~] = fitARMAGARCH(data_est(:,dim),[1 0 1 0 1],'NORMAL',startval);
           data_param(:,dim) = param_temp';
           data_resid(:,dim) = resid_temp;
           ht_est(:,dim) = ht_temp;
           data_uresid(:,dim) = tiedrank(resid_temp)/(EstimationWindowSize+1); % non-parametric transformation
           ht_one(i,dim) = data_param(3,dim) + ...
                                    data_param(4,dim) * ht_est(EstimationWindowSize,dim) * data_resid(EstimationWindowSize,dim)^2 + ...
                                    data_param(5,dim) * ht_est(EstimationWindowSize,dim); % volatility forecast
        end

        data_sim = NaN(SizeMonteCarlo,N);
        for ii = 1:SizeMonteCarlo
            for dd = 1:N
                data_sim(ii,dd) = data_param(1,dd) + ...
                                            data_param(2,dd) * data_est(EstimationWindowSize,dd) +...
                                            sqrt(ht_one(i,dd)) * data_resid_sim(ii,dd);                
            end
        end
        VaR(i,:,index) = quantile(log(1+mean(exp(data_sim)-1,2)),pVaR);
    end
    
    % 4. calculate the violation numbers/coverage rate/ kupiec test
    temp_return = log(1+mean(exp(data0)-1,2));
    I = [temp_return(TestWindow) < VaR(:,1,index) temp_return(TestWindow) < VaR(:,2,index) ];
    v(index, :) = sum(I);
    coverage(index,:) = v(index,:)/TestWindowSize;
    stat_kupiec(index,:) = [fPFtest(I(:,1), pVaR(1)) fPFtest(I(:,2), pVaR(2))];
    pval_kupiec(index,:) = [1 - chi2cdf(stat_kupiec(index,1),1) 1 - chi2cdf(stat_kupiec(index,2),1)];
end
save(strcat('Workspace_', mom, '_', simfunc_est, '_N', num2str(N),'.mat'));