% Code to simulate data from a Gaussian or Clayton copula and then estimate
% the parameter using simulated method of moments
%
%  Dong Hwan Oh and Andrew J. Patton
%
%  22 October 2013
%
% This code was developed for the following paper:
%
% Oh, D.H. and A.J. Patton, 2013, "Simulated Method of Moments Estimation
% for Copula-Based Multivariate Models," Journal of the American
% Statistical Association, 108(502), 689-700.

addpath(genpath('\\psf\Home\Documents\Copula\code_SMMFactorCopula'));  %set path to the folder that contains this code (and the helper functions)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% USER INPUTS

method = 'factor_copula' ; % 'clayton' or 'gaussian' or 'factor_copula'
T = 500 ;       % the sample size
N = 3;          % the cross-section dimension (need N>=2)

S1 = 5 ;         % number of simulations in preliminary SMM, used just to get reasonable starting values (set to zero to skip this step)
S2 = 25 ;        % nunber of simulations in main SMM, used to get the actual estimate

R = 3 ;        % number of replications (useful to see dispersion of the estimator around the true value)
iniSeed = 333;  % fixing the RNG seed so that this code always gives the same result

W = 1 ;                                 % weight matrix for SMM (here, identity matrix is used)
momfunc = 'momfunc_Equidep';            % moment functions to use in SMM estimation


theta_hat = nan(R,3);
tic;
for rr = 1:R
    
    switch lower(method)
        case 'clayton'
            simfunc = 'simfunc_Clayton';    % simulate function for SMM
            lh = 0.001; uh = 100;           % bound for parameter of Clayton
            theta0 = 2;                     % true theta
            theta_guess = 0.4;              % initial guess for true theta
        case 'gaussian'
            simfunc = 'simfunc_Gaussian';
            lh = -0.9999; uh = 0.9999;      % bound for parameter of Gaussian
            theta0 = 0.5;
            theta_guess = 0.1;
            
        case 'factor_copula'              % Factor copula: Skew-t(sig^2,nu,lambda) + t(nu)
            simfunc = 'simfunc_Factor';  % 2nd (main) stage simulating function
            lh_1st = [0.0001;-0.99];
            uh_1st = [10;0.99];
            
            lh = [0.0001;0.01;-0.99];
            uh = [10;0.49;0.99];
            
            theta0 = [1 0.25 -0.5] ;      % true theta =[ sig^2, nu_inv, lam]
            theta_guess = [2 0];          % initial guess for 1st stage [ sig^2, lam]
    end
    
    %%%%% simulate true data
    e0 = [ T, N, rr*iniSeed ];   % [ # of observation, # of variables, seed (changes, deterministically, across each replication) ]
    data = feval(simfunc,theta0,e0) ;

    
    %%%%%% to get reasonalble initial theta using only (S1 x T) simulated data %%%%%%%%%%%%%%%%%%%
    if strcmp(lower(method),'factor_copula')  % this case gets treated differently to the other two, as we do a grid search on nu to get better starting values
        if S1>=1
            Bnu_inv = [0.05,0.15,0.25,0.35,0.45] ;
            theta_ini_temp = nan(2,length(Bnu_inv));
            obj_temp = nan(length(Bnu_inv),1);
            for ii = 1:length(Bnu_inv)   % Grid search for nuinv
                ee1 = [T*S1, N, 2*rr*iniSeed, Bnu_inv(ii)];   % the last element for nuinv
                [theta_temp, objval_temp ]= fminsearchbnd('objfunc_SMM',theta_guess,lh_1st,uh_1st,[],simfunc,ee1,momfunc,data,W);
                theta_ini_temp(:,ii) = theta_temp;
                obj_temp(ii) = objval_temp;
            end
            inx = find(obj_temp==min(obj_temp));
            theta_ini = [ theta_ini_temp(1,inx); Bnu_inv(inx); theta_ini_temp(2,inx)] ;
        else
            theta_ini = theta_guess;
        end
    else
        if S1>=1
            ee1 = [T*S1, N, 2*rr*iniSeed ];   % [ # of observations for simulated data, # of variables, seed]
            theta_ini = fminsearchbnd('objfunc_SMM',theta_guess,lh,uh,[],simfunc,ee1,momfunc,data,W);
        else
            theta_ini = theta_guess;
        end
    end
    
    %%%%%% SMM estimation using (25 x T) simulated data %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    ee2 = [T*S2, N, 2*rr*iniSeed ];
    theta_SMM = fminsearchbnd('objfunc_SMM',theta_ini,lh,uh,[],simfunc,ee2,momfunc,data,W);
    
    %%%%%% stack estimates
    theta_hat(rr,1:length(theta_SMM)) = theta_SMM;
    
end
toc  
% takes about 13 seconds for N=3, T=500, R=10, S1=5, S2=25 for the clayton
% or Gaussian copulas, takes 14.7 minutes for the factor copula
