function [u raw_u] = simfunc_Factor(theta,other)
% function [u raw_u] = simfunc_Factor(theta,other)
%
% Function to simulate from a skew t-t factor copula
%
% INPUTS:   theta = [sig2,nuinv,lam] or [sig2,lam], the parameter of the copula. (The latter case is used when nu is fixed, eg during a grid search)
%           other = [T,N,seed] or [T,N,seed,nuinv]. the sample size, cross section dimension and RNG seed to use
%
% OUTPUTS:  u, a TxN matrix of the EDF-transforms of draws from the skew t-t factor copula (used in estimation)
%           raw_u, a TxN matrix of the original draws from the factor copula
%
%  Dong Hwan Oh and Andrew J. Patton
%
%  22 October 2013
%
% This code was developed for the following paper:
%
% Oh, D.H. and A.J. Patton, 2013, "Simulated Method of Moments Estimation
% for Copula-Based Multivariate Models," Journal of the American
% Statistical Association, 108(502), 689-700.


if length(theta)==3 
    sig2   = theta(1)  ;
    nu    = 1/theta(2)  ;
    lam    = theta(3) ;

    T = other(1);
    N = other(2);
    seed = other(3);    
else      % then we are in the "first stage" and are doing a grid search over nu    
    sig2 = theta(1) ;
    lam  = theta(2) ;

    T = other(1);
    N = other(2);
    seed =  other(3);
    nu = 1/other(4) ;
end

z = sqrt(sig2)*skewtdis_rnd2(nu,lam,T,seed)  ; 

eps = zeros(T,N);
for i = 1:N
    eps(:,i) = skewtdis_rnd2(nu,0,T,i*3*seed) ;
end

raw_u = z*ones(1,N) + eps ;
u  = tiedrank(raw_u)/(T+1);

 